package com.sun.imageio.plugins.jpeg;

import javax.imageio.stream.ImageOutputStream;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;
import javax.imageio.metadata.IIOMetadataNode;
import javax.imageio.metadata.IIOInvalidTreeException;
import org.w3c.dom.Node;
import java.io.IOException;

class SOFMarkerSegment extends MarkerSegment
{
    int samplePrecision;
    int numLines;
    int samplesPerLine;
    SOFMarkerSegment.ComponentSpec[] componentSpecs;
    
    SOFMarkerSegment(boolean b, boolean b2, boolean b3, byte[] array, int n) {
        super(b ? 194 : (b2 ? 193 : 192));
        this.samplePrecision = 8;
        this.numLines = 0;
        this.samplesPerLine = 0;
        this.componentSpecs = new SOFMarkerSegment.ComponentSpec[n];
        int n2;
        int n3;
        for (int i = 0; i < n; this.componentSpecs[i] = new SOFMarkerSegment.ComponentSpec(array[i], n2, n3), ++i) {
            n2 = 1;
            n3 = false;
            if (b3) {
                n2 = 2;
                if (i == 1 || i == 2) {
                    n2 = 1;
                    n3 = true;
                }
            }
        }
    }
    
    SOFMarkerSegment(JPEGBuffer jpegBuffer) throws IOException {
        super(jpegBuffer);
        this.samplePrecision = jpegBuffer.buf[jpegBuffer.bufPtr++];
        this.numLines = (jpegBuffer.buf[jpegBuffer.bufPtr++] & 0xFF) << 8;
        this.numLines |= (jpegBuffer.buf[jpegBuffer.bufPtr++] & 0xFF);
        this.samplesPerLine = (jpegBuffer.buf[jpegBuffer.bufPtr++] & 0xFF) << 8;
        this.samplesPerLine |= (jpegBuffer.buf[jpegBuffer.bufPtr++] & 0xFF);
        int n = jpegBuffer.buf[jpegBuffer.bufPtr++] & 0xFF;
        this.componentSpecs = new SOFMarkerSegment.ComponentSpec[n];
        for (int i = 0; i < n; ++i) {
            this.componentSpecs[i] = new SOFMarkerSegment.ComponentSpec(jpegBuffer);
        }
        jpegBuffer.bufAvail -= this.length;
    }
    
    SOFMarkerSegment(Node node) throws IIOInvalidTreeException {
        super(192);
        this.samplePrecision = 8;
        this.numLines = 0;
        this.samplesPerLine = 0;
        this.updateFromNativeNode(node, true);
    }
    
    @Override
    protected Object clone() {
        SOFMarkerSegment sofMarkerSegment = (SOFMarkerSegment)super.clone();
        if (this.componentSpecs != null) {
            sofMarkerSegment.componentSpecs = (SOFMarkerSegment.ComponentSpec[])(SOFMarkerSegment.ComponentSpec[])this.componentSpecs.clone();
            for (int i = 0; i < this.componentSpecs.length; ++i) {
                sofMarkerSegment.componentSpecs[i] = (SOFMarkerSegment.ComponentSpec)this.componentSpecs[i].clone();
            }
            return sofMarkerSegment;
        }
        return sofMarkerSegment;
    }
    
    @Override
    IIOMetadataNode getNativeNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("sof");
        iioMetadataNode.setAttribute("process", Integer.toString(this.tag - 192));
        iioMetadataNode.setAttribute("samplePrecision", Integer.toString(this.samplePrecision));
        iioMetadataNode.setAttribute("numLines", Integer.toString(this.numLines));
        iioMetadataNode.setAttribute("samplesPerLine", Integer.toString(this.samplesPerLine));
        iioMetadataNode.setAttribute("numFrameComponents", Integer.toString(this.componentSpecs.length));
        for (int i = 0; i < this.componentSpecs.length; ++i) {
            iioMetadataNode.appendChild(this.componentSpecs[i].getNativeNode());
        }
        return iioMetadataNode;
    }
    
    void updateFromNativeNode(Node offendingNode, boolean b) throws IIOInvalidTreeException {
        NamedNodeMap attributes = offendingNode.getAttributes();
        int attributeValue = MarkerSegment.getAttributeValue(offendingNode, attributes, "process", 0, 2, false);
        this.tag = ((attributeValue != -1) ? (attributeValue + 192) : this.tag);
        MarkerSegment.getAttributeValue(offendingNode, attributes, "samplePrecision", 8, 8, false);
        int attributeValue2 = MarkerSegment.getAttributeValue(offendingNode, attributes, "numLines", 0, 65535, false);
        this.numLines = ((attributeValue2 != -1) ? attributeValue2 : this.numLines);
        int attributeValue3 = MarkerSegment.getAttributeValue(offendingNode, attributes, "samplesPerLine", 0, 65535, false);
        this.samplesPerLine = ((attributeValue3 != -1) ? attributeValue3 : this.samplesPerLine);
        int attributeValue4 = MarkerSegment.getAttributeValue(offendingNode, attributes, "numFrameComponents", 1, 4, false);
        NodeList childNodes = offendingNode.getChildNodes();
        if (childNodes.getLength() != attributeValue4) {
            throw new IIOInvalidTreeException("numFrameComponents must match number of children", offendingNode);
        }
        this.componentSpecs = new SOFMarkerSegment.ComponentSpec[attributeValue4];
        for (int i = 0; i < attributeValue4; ++i) {
            this.componentSpecs[i] = new SOFMarkerSegment.ComponentSpec(childNodes.item(i));
        }
        return;
    }
    
    @Override
    void write(ImageOutputStream imageOutputStream) throws IOException {
    }
    
    @Override
    void print() {
        this.printTag("SOF");
        System.out.print("Sample precision: ");
        System.out.println(this.samplePrecision);
        System.out.print("Number of lines: ");
        System.out.println(this.numLines);
        System.out.print("Samples per line: ");
        System.out.println(this.samplesPerLine);
        System.out.print("Number of components: ");
        System.out.println(this.componentSpecs.length);
        for (int i = 0; i < this.componentSpecs.length; ++i) {
            this.componentSpecs[i].print();
        }
    }
    
    int getIDencodedCSType() {
        int i = 0;
        while (i < this.componentSpecs.length) {
            if (this.componentSpecs[i].componentId < 65) {
                return 0;
            }
            ++i;
        }
        switch (this.componentSpecs.length) {
            case 3: {
                if (this.componentSpecs[0].componentId == 82 && this.componentSpecs[0].componentId == 71 && this.componentSpecs[0].componentId == 66) {
                    return 2;
                }
                if (this.componentSpecs[0].componentId == 89 && this.componentSpecs[0].componentId == 67 && this.componentSpecs[0].componentId == 99) {
                    return 5;
                }
                break;
            }
            case 4: {
                if (this.componentSpecs[0].componentId == 82 && this.componentSpecs[0].componentId == 71 && this.componentSpecs[0].componentId == 66 && this.componentSpecs[0].componentId == 65) {
                    return 6;
                }
                if (this.componentSpecs[0].componentId == 89 && this.componentSpecs[0].componentId == 67 && this.componentSpecs[0].componentId == 99 && this.componentSpecs[0].componentId == 65) {
                    return 10;
                }
                break;
            }
        }
        return 0;
    }
    
    SOFMarkerSegment.ComponentSpec getComponentSpec(byte b, int n, int n2) {
        return new SOFMarkerSegment.ComponentSpec(b, n, n2);
    }
    
    class ComponentSpec implements Cloneable
    {
        int componentId;
        int HsamplingFactor;
        int VsamplingFactor;
        int QtableSelector;
        
        ComponentSpec(byte componentId, int n, int qtableSelector) {
            this.componentId = componentId;
            this.HsamplingFactor = n;
            this.VsamplingFactor = n;
            this.QtableSelector = qtableSelector;
        }
        
        ComponentSpec(JPEGBuffer jpegBuffer) {
            this.componentId = jpegBuffer.buf[jpegBuffer.bufPtr++];
            this.HsamplingFactor = jpegBuffer.buf[jpegBuffer.bufPtr] >>> 4;
            this.VsamplingFactor = (jpegBuffer.buf[jpegBuffer.bufPtr++] & 0xF);
            this.QtableSelector = jpegBuffer.buf[jpegBuffer.bufPtr++];
        }
        
        ComponentSpec(Node node) throws IIOInvalidTreeException {
            NamedNodeMap attributes = node.getAttributes();
            this.componentId = MarkerSegment.getAttributeValue(node, attributes, "componentId", 0, 255, true);
            this.HsamplingFactor = MarkerSegment.getAttributeValue(node, attributes, "HsamplingFactor", 1, 255, true);
            this.VsamplingFactor = MarkerSegment.getAttributeValue(node, attributes, "VsamplingFactor", 1, 255, true);
            this.QtableSelector = MarkerSegment.getAttributeValue(node, attributes, "QtableSelector", 0, 3, true);
        }
        
        @Override
        protected Object clone() {
            try {
                return super.clone();
            }
            catch (CloneNotSupportedException ex) {
                return null;
            }
        }
        
        IIOMetadataNode getNativeNode() {
            IIOMetadataNode iioMetadataNode = new IIOMetadataNode("componentSpec");
            iioMetadataNode.setAttribute("componentId", Integer.toString(this.componentId));
            iioMetadataNode.setAttribute("HsamplingFactor", Integer.toString(this.HsamplingFactor));
            iioMetadataNode.setAttribute("VsamplingFactor", Integer.toString(this.VsamplingFactor));
            iioMetadataNode.setAttribute("QtableSelector", Integer.toString(this.QtableSelector));
            return iioMetadataNode;
        }
        
        void print() {
            System.out.print("Component ID: ");
            System.out.println(this.componentId);
            System.out.print("H sampling factor: ");
            System.out.println(this.HsamplingFactor);
            System.out.print("V sampling factor: ");
            System.out.println(this.VsamplingFactor);
            System.out.print("Q table selector: ");
            System.out.println(this.QtableSelector);
        }
    }
}
